sign define LspDiagnosticsSignError text=✗ texthl=LspDiagnosticsSignError linehl= numhl=
sign define LspDiagnosticsSignWarning text=⚠ texthl=LspDiagnosticsSignWarning linehl= numhl=
sign define LspDiagnosticsSignInformation text=ℹ texthl=LspDiagnosticsSignInformation linehl= numhl=
sign define LspDiagnosticsSignHint text= texthl=LspDiagnosticsSignHint linehl= numhl=

" nvim-lspconfig
set completeopt=menuone,noinsert,noselect

nnoremap <silent> gD    <cmd>lua vim.lsp.buf.declaration()<CR>
nnoremap <silent> gd    <cmd>lua vim.lsp.buf.definition()<CR>
nnoremap <silent> K     <cmd>lua vim.lsp.buf.hover()<CR>
nnoremap <silent> gi    <cmd>lua vim.lsp.buf.implementation()<CR>
nnoremap <silent> <c-k> <cmd>lua vim.lsp.buf.signature_help()<CR>
nnoremap <silent> <space>wa <cmd>lua vim.lsp.buf.add_workspace_folder()<CR>
nnoremap <silent> <space>wr <cmd>lua vim.lsp.buf.remove_workspace_folder()<CR>
nnoremap <silent> <space>wl <cmd>lua print(vim.inspect(vim.lsp.buf.list_workspace_folders()))<CR>
nnoremap <silent> <space>D  <cmd>lua vim.lsp.buf.type_definition()<CR>
nnoremap <silent> <space>rn <cmd>lua vim.lsp.buf.rename()<CR>
nnoremap <silent> gr    <cmd>lua vim.lsp.buf.references()<CR>
nnoremap <silent> g0    <cmd>lua vim.lsp.buf.document_symbol()<CR>
nnoremap <silent> gW    <cmd>lua vim.lsp.buf.workspace_symbol()<CR>
nnoremap <silent> <space>e <cmd>lua vim.lsp.diagnostic.show_line_diagnostics()<CR>
nnoremap <silent> [d       <cmd>lua vim.lsp.diagnostic.goto_prev()<CR>
nnoremap <silent> ]d       <cmd>lua vim.lsp.diagnostic.goto_next()<CR>
nnoremap <silent> <space>q <cmd>lua vim.lsp.diagnostic.set_loclist()<CR>

lua << EOF
local nvim_lsp = require('lspconfig')

local on_attach = function(client, bufnr)
    vim.api.nvim_command('autocmd CursorHold <buffer> lua vim.lsp.diagnostic.show_line_diagnostics()')
    vim.lsp.handlers["textDocument/publishDiagnostics"] = vim.lsp.with(
        vim.lsp.diagnostic.on_publish_diagnostics, {virtual_text=false}
    )
end

local settings = {
    pylsp = {
        plugins = {
            pycodestyle = {maxLineLength = 79},
            pylint = {
                enabled = true,
                args = {
                    "--max-line-length=79",
                    "--disable=missing-docstring,invalid-name,redefined-outer-name,logging-format-interpolation"
                }
            },
            pylsp_mypy = {enabled = true},
        }
    }
}

local servers = {'pylsp', 'clangd'}
for _, lsp in ipairs(servers) do
    nvim_lsp[lsp].setup{on_attach=on_attach, settings={[lsp]=settings[lsp]}}
end
EOF

" completion-nvim
autocmd FileType python,c lua require('completion').on_attach()
let g:completion_matching_strategy_list = ['exact', 'substring', 'fuzzy']
let g:completion_matching_ignore_case = 1  " not v:true, 1. ¯\_(ツ)_/¯
let g:completion_matching_smart_case = 1  " not v:true, 1. ¯\_(ツ)_/¯
let g:completion_trigger_on_delete = 1  " not v:true, 1. ¯\_(ツ)_/¯

" lspkind-nvim
lua << EOF
local symbol_map = require('lsp_symbols')
require('lspkind').init({with_text=false, symbol_map=symbol_map})
EOF
